const { formatMessage } = require('./formatters')

/**
 * Normalize Gitea webhook payload to common format
 * @param {Object} payload - Gitea webhook payload
 * @returns {Object} - Normalized webhook data
 */
const normalizeGiteaPayload = (payload) => {
    // Validate required fields
    if (!payload?.repository?.full_name || !payload?.ref) {
        return {
            repository: {
                full_name: payload?.repository?.full_name || 'unknown',
                html_url: payload?.repository?.html_url || payload?.repository?.url || '#',
                id: payload?.repository?.id || 0
            },
            ref: payload?.ref || 'unknown',
            commits: []
        }
    }

    // Handle missing or invalid commits array
    const commits = Array.isArray(payload.commits) ? payload.commits : []

    return {
        repository: {
            full_name: payload.repository.full_name,
            html_url: payload.repository.html_url || payload.repository.url,
            id: payload.repository.id || 0
        },
        ref: payload.ref,
        commits: commits.map(commit => ({
            id: commit.id || `${Date.now()}-${Math.random()}`,
            message: commit.message || 'No message',
            stats: {
                additions: commit.added?.length || 0,
                deletions: commit.removed?.length || 0
            }
        }))
    }
}

/**
 * Handle Gitea webhook
 * @param {Object} payload - Webhook payload
 * @param {Set} reportedCommits - Set of reported commit hashes
 * @param {Map} commitTimestamps - Map of commit timestamps
 * @returns {Object} - Formatted message or null
 */
const handleGitea = (payload, reportedCommits, commitTimestamps) => {
    const data = normalizeGiteaPayload(payload)
    
    // Filter new commits
    const newCommits = data.commits.filter(commit => {
        if (reportedCommits.has(commit.id)) return false
        reportedCommits.add(commit.id)
        commitTimestamps.set(commit.id, Date.now())
        return true
    })

    if (newCommits.length === 0) return null

    return formatMessage(data, newCommits)
}

module.exports = { handleGitea } 